 /*
  * helpers.c
  * Copyright (C) Aitor Cuadrado Zubizarreta <aitor_czr@gnuinos.org>
  * 
  * simple-netaid is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  * 
  * simple-netaid is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  * See the GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License along
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  * 
  * See the COPYING file.
  */

#ifndef _GNU_SOURCE
   #define _GNU_SOURCE 1
#endif

// #include <sys/wait.h>
#include <sys/stat.h>
#include <assert.h>
#include <pstat/libpstat.h>

#include "def.h"
#include "helpers.h"
#include "sbuf.h"
#include "interfaces.h"
#include "ipaddr.h"
#include "wireless.h"

#define MAX_SIZE 128
#define ARRAY_DIM 6

// Usage: strncpy_t(str1, sizeof(str1), str2, strlen(str2));
// Copy string "in" with at most "insz" chars to buffer "out", which
// is "outsz" bytes long. The output is always 0-terminated. Unlike
// strncpy(), strncpy_t() does not zero fill remaining space in the
// output buffer:
static
char* strncpy_t(char* out, size_t outsz, const char* in, size_t insz)
{
   assert(outsz > 0);
   while(--outsz > 0 && insz > 0 && *in) { *out++ = *in++; insz--; }
   *out = 0;
   return out;
}

void print_active_wifis(const char *wireless_device)
{
   sbuf_t s;
   sbuf_init(&s);   
   scan_active_wifis(wireless_device, &s);
   printf("%s\n", s.buf);
   sbuf_free(&s);
}

short ifquery(const char *ifname)
{
   FILE *fp;
   short state;
   sbuf_t cmd;
      
   sbuf_init(&cmd);
   sbuf_concat(&cmd, 2, "/sbin/ifquery --state ", ifname);
   fp = popen(cmd.buf, "r");
   if(!fp) {
      state=-1;
      goto Free;
   }
   
   state=0;
   if(fgetc(fp)!=EOF) state=1;
   
   pclose(fp);
   
Free:
   sbuf_free(&cmd);
   
   return state;
}

void kill_all_processes()
{
	DIR *dirp = NULL;
	struct dirent *dir = NULL;
    const char *processes[ARRAY_DIM] = { "dhclient", "dhcpcd", "udhcpc", "pump", "dhcp6c", "wpa_supplicant" };
	
	dirp=opendir("/proc");
	if(!dirp) {
		printf("\n\tERROR: unable to open /proc for read\n\n");
		exit(EXIT_FAILURE);
	}
	
	while((dir=readdir(dirp)) != NULL ) {

        int rc = 0;
	    struct stat sb;
        char *proc_cmd = NULL;
        bool is_uint_ok = true;
        char pth[MAX_SIZE] = {0};
	    
	    for(unsigned i=0; i<strlen(dir->d_name); i++) {
		   if(!isdigit(dir->d_name[i])) is_uint_ok = false;
	    }
        
        if(!is_uint_ok) continue;
        
        strncpy_t(pth, sizeof(pth), "/proc/", strlen("/proc/"));
        strcat(pth, dir->d_name);
        if (stat(pth, &sb) == -1) {
           perror("stat");
           exit(EXIT_FAILURE);
        }
        
        if(!lstat(dir->d_name, &sb)) continue;
 
        /* Only consider directories */
        if(!S_ISDIR(sb.st_mode)) continue;

		rc = pstat_get_binary_path_from_string(&proc_cmd, MAX_SIZE, dir->d_name);
		if(rc) continue;

        for(unsigned int i=0; i<ARRAY_DIM; i++) {		                    
           if(!strcmp(rindex(proc_cmd, '/') + 1, processes[i])) {
              if( (kill(atoi(dir->d_name), SIGTERM)) != 0 ) kill(atoi(dir->d_name), SIGKILL);
           }
        }

	    free(proc_cmd);
	}
	
    closedir(dirp);
}
